using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using TMPro;

[RequireComponent(typeof(Outline))]

public class ScanComponent : MonoBehaviour
{
    // Start is called before the first frame update
    public UIAboveHeadStatusEnum uiAboveHeadStatus = new UIAboveHeadStatusEnum();
    //SwitchChecks for better performance in update function
    private UIAboveHeadStatusEnum uiAboveHeadStatusSwitchCheck;
    public enum UIAboveHeadStatusEnum
    {
        Verified,
        Unverified,
        SuspiciousIdentity
    };

    [SerializeField]
    private string avatarName;

    public UIAboveHeadModeEnum uiAboveHeadMode = new UIAboveHeadModeEnum();
    //the logic for uiAboveHeadMode is currently very specific, so this switch is too general and not needed
    //private UIAboveHeadModeEnum uiAboveHeadModeSwitchCheck;
    public enum UIAboveHeadModeEnum
    {
        Disabled,
        NameOnly,
        SignOnly,
        NameAndSign
    };

    [Range(0.0f, 1f)]
    public float opacity = 1f;
    [Range(0.0f, 1f)]
    public float saturation = 1f;

    public ScanEnum scanType = new ScanEnum();
    private ScanEnum scanTypeSwitchCheck;
    public enum ScanEnum
    {
        Disabled,
        Loop,
        Once
    };

    public ScanModeEnum scanMode = new ScanModeEnum();
    private ScanModeEnum scanModeSwitchCheck;
    public enum ScanModeEnum
    {
        Mode1,
        Mode2
    };

    [Range(0.0f, 1f)]
    public float speed = 1f;

    public bool showOutline;
    private Outline outline;

    //private variables

    private Animator thisAnimator;
    private List<Transform> bodyNodes;

    private List<Material> playermats;

    //UIAboveHead variables
    private List<GameObject> UI_Prefabs;
    private int activeUI;
    private Transform HeadAnchor;
    private List<GameObject> generatedUIsAboveHead;
    private TextMeshProUGUI textMeshPro;
    private float offsetUIWhenNameActive;
    private Camera LookAtCam;
    private GameObject parentForLookAt;

    //shader variables
    private float decreasingScanThickness;
    private float scanHeight;
    private float step;

    //colors based on Status
    private Color colorBasedOnStatus;
    private Color greenColor;
    private Color redColor;
    private Color greyColor;
    void Start()
    {
        //load bodyNodes
        thisAnimator = GetComponent<Animator>();
        bodyNodes = new List<Transform>();
        SkinnedMeshRenderer[] meshes = gameObject.GetComponentsInChildren<SkinnedMeshRenderer>();
        InitializeBodyNodes();
       
        //load avatar materials
        playermats = new List<Material>();
        for (int i = 0; i < meshes.Length; i++)
        {
            playermats.Add(meshes[i].material);
        }

        //scan = scanHeight;//in % from 0 to 1 -> 0 to 100
        scanHeight = 0.0f;
        step = 0.01f;
        decreasingScanThickness = 0.3f;
        foreach (Material mat in playermats)
        {
            mat.SetFloat("_Scan", scanHeight);
            mat.SetFloat("_MomentaryBodyHeight", GetHighestBodyNodesTransform().position.y - GetLowestBodyNodesTransform().position.y + 0.3f);
            mat.SetFloat("_LowestWorldPosition", GetLowestBodyNodesTransform().position.y - 0.05f);
            mat.SetColor("_ScanColor", colorBasedOnStatus);
        }

        if (LookAtCam == null)
        {
            if(Camera.main == null)
            {
                Camera[] allCameras = GameObject.FindObjectsOfType<Camera>();
                if(allCameras.Length == 0)
                {
                    //assign self gameobject
                    //LookAtCam = gameObject;
                    Debug.LogError("No Camera found!");
                    //popup window
                }
                else
                {
                    LookAtCam = allCameras[0];
                }
            }
            else
            {
                LookAtCam = Camera.main;
            }
            
        }

        //Load AboveHeadUI
        InitializeUIAboveHeadPrefabs();
        InitializeGeneratedUIsAboveHead();

        HeadAnchor = thisAnimator.GetBoneTransform(HumanBodyBones.Head);
        //LookAtCamera

        //initialize enums
        scanTypeSwitchCheck = ScanEnum.Disabled;
        uiAboveHeadStatusSwitchCheck = UIAboveHeadStatusEnum.Verified;
        scanModeSwitchCheck = ScanModeEnum.Mode1;
        //the logic for uiAboveHeadMode is currently very specific, so this switch is too general and not needed
        //uiAboveHeadModeSwitchCheck = UIAboveHeadModeEnum.Disabled;

        DeactivateAllUIAbovehead();

        //load outline plugin and change certain settings
        outline = GetComponent<Outline>();
        outline.enabled = false;
        outline.OutlineWidth = 4.0f;
        outline.OutlineMode = Outline.Mode.OutlineVisible;

        //colors for colorBasedOnStatus
        greenColor = new Color(0f, 1f, 0f, 0.5f);
        redColor = new Color(1f, 0f, 0f, 0.5f);
        greyColor =  new Color(0.5f, 0.5f, 0.5f, 0.5f);

        parentForLookAt = new GameObject();
    }

    // Update is called once per frame
    void Update()
    {
        //if either the value for scanType or scanMode is switched, reset all important variables 
        if(scanType != scanTypeSwitchCheck || scanMode != scanModeSwitchCheck)
        {
            scanTypeSwitchCheck = scanType;
            scanModeSwitchCheck = scanMode;
            scanHeight = 0.0f;
            step = 0.01f;
            decreasingScanThickness = 0.3f;
        }

        //if the value for uiAboveHeadStatus is switched, deactivate all UIAboveHead and activate the right one in a later if statement
        if (uiAboveHeadStatus != uiAboveHeadStatusSwitchCheck)
        {
            uiAboveHeadStatusSwitchCheck = uiAboveHeadStatus;
            DeactivateAllUIAbovehead();
        }

        //activate the according UIAboveHead and set colorBasedOnStatus to the according color
        if (uiAboveHeadStatus == UIAboveHeadStatusEnum.Verified)
        {
            activeUI = 3;
            colorBasedOnStatus = greenColor;
        }
        else if (uiAboveHeadStatus == UIAboveHeadStatusEnum.Unverified)
        {
            activeUI = 2;
            colorBasedOnStatus = redColor;
        }
        else if (uiAboveHeadStatus == UIAboveHeadStatusEnum.SuspiciousIdentity)
        {
            activeUI = 1;
            colorBasedOnStatus = greyColor;
        }

        //the logic for uiAboveHeadMode is currently very specific, so this switch is too general and not needed
        /*if (uiAboveHeadMode != uiAboveHeadModeSwitchCheck)
        {
            uiAboveHeadModeSwitchCheck = uiAboveHeadMode;
        }*/

        //if any uiAboveHeadMode that shows the name is chosen, show the avatarName / else don't show it
        if ((uiAboveHeadMode == UIAboveHeadModeEnum.NameAndSign || uiAboveHeadMode == UIAboveHeadModeEnum.NameOnly) && avatarName != "")
        {
            //generatedUIsAboveHead[0] is always the name
            generatedUIsAboveHead[0].SetActive(true);
            textMeshPro = generatedUIsAboveHead[0].GetComponentInChildren<TextMeshProUGUI>();
            textMeshPro.text = avatarName;
            //offset the active UIAboveHead sign to the left of the avatarName
            offsetUIWhenNameActive = textMeshPro.GetRenderedValues().x / 2.0f + 0.1f;
        }
        else
        {
            generatedUIsAboveHead[0].SetActive(false);
            offsetUIWhenNameActive = 0.0f;
        }

        //if any uiAboveHeadMode that shows the sign is chosen, show the sign / else don't show it 
        if (uiAboveHeadMode == UIAboveHeadModeEnum.NameAndSign || uiAboveHeadMode == UIAboveHeadModeEnum.SignOnly)
        {
            generatedUIsAboveHead[activeUI].SetActive(true);
        }
        else
        {
            generatedUIsAboveHead[activeUI].SetActive(false);
        }

        //rotate active UIAboveHead gameobjects so they face the camera looking at it
        RotateUIAboveHeadTowardsCamera();

        //show or hide outline plugin
        if (showOutline)
        {
            outline.enabled = true;
            outline.OutlineColor = colorBasedOnStatus;
        }
        else
        {
            outline.enabled = false;
        }

        //Scan Loop
        if (scanType == ScanEnum.Loop)
        {
            //Mode1: Scan goes up and down
            if(scanMode == ScanModeEnum.Mode1)
            {
                scanHeight += (Time.deltaTime * 100 * step * speed);
                //update shader variables for every material on the player
                foreach (Material mat in playermats)
                {
                    //this is a boolean (1 -> true)
                    mat.SetInt("_useScan", 1);

                    mat.SetFloat("_Scan", scanHeight);
                    mat.SetFloat("_ScanThickness", 1f);
                    mat.SetColor("_ScanColor", colorBasedOnStatus);
                    mat.SetFloat("_MomentaryBodyHeight", GetHighestBodyNodesTransform().position.y - GetLowestBodyNodesTransform().position.y + 0.3f);
                    mat.SetFloat("_LowestWorldPosition", GetLowestBodyNodesTransform().position.y - 0.05f);
                }

                //going up or down is controlled by the "step" variable being positive or negative
                if (scanHeight > 1) step = -0.01f;
                else if (scanHeight < 0) step = 0.01f;
            } 
            //Mode2: Scan only goes up
            else if(scanMode == ScanModeEnum.Mode2)
            {
                //if scan < 1 just change the scan variable (controls scan height)
                if (scanHeight < 1) scanHeight += (Time.deltaTime * 100 * step * speed);
                else
                {
                    //when scan == 1 (so you are at the top of the avatar) -> let scan (scanheight) stay at the top and decrease the scan thickness to make it look like it is still moving up
                    //this "trick" has to be used because scan (scanheight) is in percent(%) and can't go above 1(100%)
                    if (decreasingScanThickness > 0) decreasingScanThickness -= (Time.deltaTime * 100 * step * speed);
                    //reset scan realated variables when finished to start again at the bottom of avatar
                    else
                    {
                        scanHeight = 0.0f;
                        step = 0.01f;
                        decreasingScanThickness = 0.3f;
                    }
                }
                //update shader variables for every material on the player
                foreach (Material mat in playermats)
                {
                    //this is a boolean
                    mat.SetInt("_useScan", 1);

                    mat.SetFloat("_Scan", scanHeight);
                    mat.SetFloat("_ScanThickness", decreasingScanThickness);
                    mat.SetColor("_ScanColor", colorBasedOnStatus);
                    mat.SetFloat("_MomentaryBodyHeight", GetHighestBodyNodesTransform().position.y - GetLowestBodyNodesTransform().position.y + 0.3f);
                    mat.SetFloat("_LowestWorldPosition", GetLowestBodyNodesTransform().position.y - 0.05f);
                }
            }
            
        }
        //Scan Once
        else if(scanType == ScanEnum.Once)
        {
            //all logic is the same as above(Scan Loop) except for looping. The scan gets disabled after running one time
            if (scanMode == ScanModeEnum.Mode1)
            {
                scanHeight += (Time.deltaTime * 100 * step * speed);
                foreach (Material mat in playermats)
                {
                    mat.SetInt("_useScan", 1);
                    mat.SetFloat("_Scan", scanHeight);
                    mat.SetFloat("_ScanThickness", 1f);
                    mat.SetColor("_ScanColor", colorBasedOnStatus);
                    mat.SetFloat("_MomentaryBodyHeight", GetHighestBodyNodesTransform().position.y - GetLowestBodyNodesTransform().position.y + 0.3f);
                    mat.SetFloat("_LowestWorldPosition", GetLowestBodyNodesTransform().position.y - 0.05f);
                }

                if (scanHeight > 1) step = -0.01f;
                else if (scanHeight < 0)
                {
                    scanType = ScanEnum.Disabled;
                }
            }
            else if (scanMode == ScanModeEnum.Mode2)
            {
                if (scanHeight < 1) scanHeight += (Time.deltaTime * 100 * step * speed);
                else
                {
                    if (decreasingScanThickness > 0) decreasingScanThickness -= (Time.deltaTime * 100 * step * speed);
                    else scanType = ScanEnum.Disabled;
                }
                foreach (Material mat in playermats)
                {
                    mat.SetInt("_useScan", 1);
                    mat.SetFloat("_Scan", scanHeight);
                    mat.SetFloat("_ScanThickness", decreasingScanThickness);
                    mat.SetColor("_ScanColor", colorBasedOnStatus);
                    mat.SetFloat("_MomentaryBodyHeight", GetHighestBodyNodesTransform().position.y - GetLowestBodyNodesTransform().position.y + 0.3f);
                    mat.SetFloat("_LowestWorldPosition", GetLowestBodyNodesTransform().position.y - 0.05f);
                }
            }
            
        }
        else if(scanType == ScanEnum.Disabled)
        {
            foreach (Material mat in playermats)
            {
                //this is a boolean (0 -> false)
                mat.SetInt("_useScan", 0);
            }
        }

        //update opacity and saturation shader variables for every material on the player
        foreach (Material mat in playermats)
        {
            mat.SetFloat("_Opacity", opacity);
            mat.SetFloat("_Saturation", saturation);
        }
    }

    //"UI_NAME 1" has to be the first gameobject to be added to UI_Prefabs because other code depends on it being at [0]
    //Load UIAboveHead prefabs -> they all have a "1" in the end because they were copied from another folder, because they need to be in the "Resources"-folder to use the "Load" function
    private void InitializeUIAboveHeadPrefabs()
    {
        UI_Prefabs = new List<GameObject>();

        //name needs to be first element to be added
        UI_Prefabs.Add(Resources.Load<GameObject>("UIs/UI_NAME 1"));

        UI_Prefabs.Add(Resources.Load<GameObject>("UIs/UI_questionmark 1"));
        UI_Prefabs.Add(Resources.Load<GameObject>("UIs/UI_cross 1"));
        UI_Prefabs.Add(Resources.Load<GameObject>("UIs/UI_check 1"));
    }

    //adds instances of uisAboveHead gameobjects to generatedUIsAboveHead list
    private void InitializeGeneratedUIsAboveHead()
    {
        generatedUIsAboveHead = new List<GameObject>();

        foreach (GameObject i in UI_Prefabs)
        {
            if (i != null)
            {
                generatedUIsAboveHead.Add(Instantiate(i, transform));
            }
        }
    }

    private void ParentInitalizedUIs()
    {
        foreach (GameObject i in generatedUIsAboveHead)
        {
            if (i != null)
            {
                //i.gameObject.transform.set
            }
        }
    }

    //adds transforms from certain HumanBodyBones to bodyNodes list
    private void InitializeBodyNodes()
    {
        bodyNodes.Add(thisAnimator.GetBoneTransform(HumanBodyBones.Head));
        bodyNodes.Add(thisAnimator.GetBoneTransform(HumanBodyBones.LeftToes));
        bodyNodes.Add(thisAnimator.GetBoneTransform(HumanBodyBones.RightToes));
        bodyNodes.Add(thisAnimator.GetBoneTransform(HumanBodyBones.LeftHand));
        bodyNodes.Add(thisAnimator.GetBoneTransform(HumanBodyBones.RightHand));
    }

    //returns lowest transform from bodyNodes list
    private Transform GetLowestBodyNodesTransform()
    {
        Transform lowestTransform = bodyNodes[0];

        foreach (Transform t in bodyNodes)
        {
            if (t.position.y < lowestTransform.position.y)
            {
                lowestTransform = t;
            }
        }

        return lowestTransform;
    }

    //returns highest transform from bodyNodes list
    private Transform GetHighestBodyNodesTransform()
    {
        Transform highestTransform = bodyNodes[0];

        foreach (Transform t in bodyNodes)
        {
            if (t.position.y > highestTransform.position.y)
            {
                highestTransform = t;
            }
        }

        return highestTransform;
    }

    //Deactivates all UIAboveHeadElements
    private void DeactivateAllUIAbovehead()
    {
        foreach (GameObject i in generatedUIsAboveHead)
        {
            i.SetActive(false);
        }
    }

    //make that the UIAboveHead always faces the camera
    private void RotateUIAboveHeadTowardsCamera()
    {
        //generatedUIs[0] is the TextMesh name
        generatedUIsAboveHead[0].transform.position = HeadAnchor.position;
        generatedUIsAboveHead[0].transform.localPosition += new Vector3(0, 0.3f, 0);

        //Vector3 v = LookAtCam.transform.position - generatedUIs[0].transform.position;
        //v.x = v.z = 0.0f;

        //better for vr
        generatedUIsAboveHead[0].transform.LookAt(LookAtCam.transform); 
        //generatedUIsAboveHead[0].transform.rotation =(LookAtCam.transform.rotation);
        //generatedUIsAboveHead[0].transform.Rotate(0,180,0);

        //only change the rotation of the currently active UIAboveHead
        generatedUIsAboveHead[activeUI].transform.position = generatedUIsAboveHead[0].transform.position + generatedUIsAboveHead[0].transform.right * offsetUIWhenNameActive;
        generatedUIsAboveHead[activeUI].transform.rotation = generatedUIsAboveHead[0].transform.rotation;
    }

}
