#include "UnityCG.cginc"

uniform float4 _LightColor0;
uniform sampler2D _MainTex;
float4 _MainTex_ST; //for tiling
uniform sampler2D _NormalMap;
uniform sampler2D _SpecularMap;
uniform float4 _frontColor;
uniform float4 _backColor;
uniform float _ambientIntensity;
uniform float _diffuseIntensity;
uniform float _specularIntensity;
uniform float _shininess;
uniform int _useLighting;
uniform int _useSpecularMap;
uniform int _useTexture;
uniform int _useVertexColor;
uniform int _useNormalMap;
uniform int _useScan;


//----------------------------------------------------------------------------------------------

struct appdata {
    float4 vertex : POSITION;
    float3 normal : NORMAL;
    float2 uv : TEXCOORD0;
    float4 color : COLOR;
};
struct v2g {
    float3 uv: TEXCOORD0;
    float4 vertex: SV_POSITION;
    float4 worldPos: TEXCOORD1;
    float4 localHeightPos : TEXCOORD2;
    float3 normal : NORMAL;///////
    float4 color : COLOR;////
};

struct g2f {
    float3 uv: TEXCOORD0;
    float4 vertex: SV_POSITION;
    float3 dist: TEXCOORD1;
    float4 worldPos:TEXCOORD2;///////
    float4 localHeightPos : TEXCOORD3;///////
    float3 normal : NORMAL;///////
    float4 color : COLOR;/////
};

struct v2f {
    float4 pos : POSITION;
    float3 normal : NORMAL;
    float2 uv : TEXCOORD0;
    float4 posWorld : TEXCOORD1;
    float4 localHeightPos : TEXCOORD2;
    float4 color : COLOR;
};

//----------------------------------------------------------------------------------------------
void phong_lighting_g2f(float3 normal,
                    float4 material,
                    float3 color,
                    g2f i,
                    inout float3 result,
                    bool useAmbient)
            {
                if(useAmbient)
                {
                    //ambient
                    result = float3(0, 0, 0);
                    if(_useTexture) {
                        result += material[0] * color * tex2D(_MainTex, i.uv).xyz;
                    }
                    else
                    {
                        result += material[0] * color;
                    }
                }

                //diffuse 
                float3 L = _WorldSpaceLightPos0.xyz - i.worldPos.xyz * _WorldSpaceLightPos0.w;
                float NL = dot(normal, L);
                if(NL > 0)
                {
                    if(_useTexture)
                    {
                        result += NL * material[1] * color * tex2D(_MainTex, i.uv).xyz * _LightColor0.rgb;
                    }
                    else
                    {
                        result += NL * material[1] * _LightColor0.rgb;
                    }
                    
                }
                
                //specular
                float3 R = normalize(reflect(-L, normal));
                float3 V = normalize(_WorldSpaceCameraPos - i.worldPos.xyz);
                float RV = dot(R, V);
                if(RV > 0)
                {
                        if(_useSpecularMap && _useTexture)
                        {
                            float3 spec_tmp = tex2D(_SpecularMap, i.uv).xyz;
                            result += spec_tmp * material[2] * pow(RV, material[3]);
                        }
                        else
                        {
                            float spec_col = material[2] * pow(RV, material[3]);
                            result += float3(spec_col, spec_col, spec_col);
                        }
                }
            }

//----------------------------------------------------------------------------------------------

            float3x3 cotangent_frame(float3 N, float3 p, float2 uv)
            {
                float3 dp1 = ddx(p);
                float3 dp2 = ddy(p);
                float2 duv1 = ddx(uv);
                float2 duv2 = ddy(uv);

                float3 dp2perp = cross(dp2, N);
                float3 dp1perp = cross(N, dp1);
                float3 T = dp2perp * duv1.x + dp1perp * duv2.x;
                float3 B = dp2perp * duv1.y + dp1perp * duv2.y;

                float invmax = rsqrt(max(dot(T,T), dot(B, B)));

                return float3x3(T * invmax, B * invmax, N);
            }

//----------------------------------------------------------------------------------------------

           float3 perturb_normal(float3 N, float3 V, float2 uv)
           {
                float3 map = (2.0 * tex2D(_NormalMap, uv).xyz) - 1.0;
                //map = -map * 255./127. - 128./127.;

                float3x3 TBN = cotangent_frame(N, -V, uv);
                return normalize(mul(TBN, map)); 
           } 


void phong_lighting(float3 normal,
                    float4 material,
                    float3 color,
                    v2f i,
                    inout float3 result,
                    bool useAmbient)
            {
                if(useAmbient)
                {
                    //ambient
                    result = float3(0, 0, 0);
                    if(_useTexture) {
                        result += material[0] * color * tex2D(_MainTex, i.uv).xyz;
                    }
                    else
                    {
                        result += material[0] * color;
                    }
                }

                //diffuse 
                float3 L = _WorldSpaceLightPos0.xyz - i.posWorld.xyz * _WorldSpaceLightPos0.w;
                float NL = dot(normal, L);
                if(NL > 0)
                {
                    if(_useTexture)
                    {
                        result += NL * material[1] * color * tex2D(_MainTex, i.uv).xyz * _LightColor0.rgb;
                    }
                    else
                    {
                        result += NL * material[1] * _LightColor0.rgb;
                    }
                    
                }
                
                //specular
                float3 R = normalize(reflect(-L, normal));
                float3 V = normalize(_WorldSpaceCameraPos - i.posWorld.xyz);
                float RV = dot(R, V);
                if(RV > 0)
                {
                        if(_useSpecularMap && _useTexture)
                        {
                            float3 spec_tmp = tex2D(_SpecularMap, i.uv).xyz;
                            result += spec_tmp * material[2] * pow(RV, material[3]);
                        }
                        else
                        {
                            float spec_col = material[2] * pow(RV, material[3]);
                            result += float3(spec_col, spec_col, spec_col);
                        }
                }
            }

//----------------------------------------------------------------------------------------------
/* 
            float3x3 cotangent_frame(float3 N, float3 p, float2 uv)
            {
                float3 dp1 = ddx(p);
                float3 dp2 = ddy(p);
                float2 duv1 = ddx(uv);
                float2 duv2 = ddy(uv);

                float3 dp2perp = cross(dp2, N);
                float3 dp1perp = cross(N, dp1);
                float3 T = dp2perp * duv1.x + dp1perp * duv2.x;
                float3 B = dp2perp * duv1.y + dp1perp * duv2.y;

                float invmax = rsqrt(max(dot(T,T), dot(B, B)));

                return float3x3(T * invmax, B * invmax, N);
            }

//----------------------------------------------------------------------------------------------

           float3 perturb_normal(float3 N, float3 V, float2 uv)
           {
                float3 map = (2.0 * tex2D(_NormalMap, uv).xyz) - 1.0;
                //map = -map * 255./127. - 128./127.;

                float3x3 TBN = cotangent_frame(N, -V, uv);
                return normalize(mul(TBN, map)); 
           } 
 */
//----------------------------------------------------------------------------------------------