﻿// Upgrade NOTE: replaced '_LightMatrix0' with 'unity_WorldToLight'

Shader "Avatar/AvatarVisualIndicator"
{
        Properties {
        ///////////for opacity and saturation////////
        _Opacity("Opacity", Range(0,1)) = 1
        _Saturation("Saturation", Range(0,1)) = 1
        ///////////for scan/////
        [Toggle] _useScan("Use Scan", int) = 0
        [HDR]_WireColor ("WireColor", Color) = (1, 0, 0, 0.5)
        [HDR]_FillColor ("FillColor", Color) = (1, 1, 1, 0)
        _WireWidth ("WireWidth", Range(0, 0.005)) = 0.001
        _Scan ("Scan", Range(0, 1)) = 1
        _ScanColor ("ScanColor", Color) = (1, 0, 0, 0.5)
        _ScanThickness ("ScanThickness", Range(0, 1)) = 1
        _BlendWidth ("BlendWidth", Range(0, 2)) = 0.3
        _LowestWorldPosition ("Lowest world position of bones", float) = 0.0
        _MomentaryBodyHeight ("Total Body height at this frame", float) = 0.0
        ////////////////////////
        _MainTex ("RGB Texture", 2D) = "white" {}
        _NormalMap ("Normals", 2D) = "white" {}
        _SpecularMap ("Specular", 2D) = "white" {}
        _frontColor ("Front Color", Color) = (1,1,1,1)
        _backColor ("Back Color", Color) = (0.5, 0.5, 0.0, 1)
        _ambientIntensity ("Ambient light intensity", float) = 0.2
        _diffuseIntensity ("Diffuse light intensity", float) = 0.7
        _specularIntensity ("Specular light intensity", float) = 0
        _shininess ("Shininess", range(0, 10)) = 0
        [Toggle] _useLighting ("Use Lighting", int) = 1
        [Toggle] _useTexture ("Use Texture", int) = 1
        [Toggle] _useVertexColor ("Use Vertex Color", int) = 0
        [Toggle] _useSpecularMap ("Use Specular Map", int) = 0
        [Toggle] _useNormalMap ("Use Normal Map", int) = 0
    }

    SubShader {
        //Tags {"RenderType" = "Opaque" }
        Tags {"Queue"="Transparent" "IgnoreProjector"="True" "RenderType" = "Opaque"}

        Pass{ //to fix rendering queue
			ZWrite On
			ColorMask 0
		}
       
        Pass{
        // an addition pass for the shadow, now the fragment that cast shadow will be clipped according to "_Scan"
        // but the shadow will be cut with a sharp edge.
            Tags{"LightMode" = "ShadowCaster"}
            
            CGPROGRAM
            #pragma vertex vert  
            #pragma fragment frag  
            #include "AvatarVisualIndicator.cginc" 


            float _Scan, _MomentaryBodyHeight, _ScanThickness, _LowestWorldPosition;
            v2f vert(appdata v)
            {
                v2f o;
                o.posWorld = mul(unity_ObjectToWorld, v.vertex);
                o.normal = normalize(mul(float4(v.normal, 0.0), unity_WorldToObject).xyz);
                o.pos = UnityObjectToClipPos(v.vertex);
                o.localHeightPos = v.vertex;
                o.uv = TRANSFORM_TEX(v.uv, _MainTex);
                o.color = v.color;

                return o;
            }

            fixed4 frag(v2f i, fixed facing : VFACE) : COLOR
            {
                fixed4 col = fixed4(0, 0, 0, 0);
                //clip(i.posWorld.y-_Scan); //fragment that are lower then _Scan will be clipped
                //clip(i.localHeightPos.y - (_Scan * 2) + 1); //fragment that are lower then _Scan will be clipped
                //
                if(_useScan){
                    float localHeightPos = i.posWorld.y - _LowestWorldPosition;

                    float scanSize = _MomentaryBodyHeight * _ScanThickness;
                    float currentScanHeight = _Scan * _MomentaryBodyHeight;

                    float localScanHeight = localHeightPos - currentScanHeight;

                    //float materialChangePos = localHeightPos - currentScanHeight;
                    //float spaceUnderScan = materialChangePos - scanSize;

                    if (localHeightPos > currentScanHeight - scanSize && localHeightPos < currentScanHeight)
                    {
                        discard;
                    }
                }
                    
                //if(localHeightPos < localScanHeight)
                    
                //clip(localHeightPos - currentScanHeight); //fragment that are lower then _Scan will be clipped
                return col;
            }
            ENDCG
        }


        Pass {
            Tags {"LightMode" = "ForwardBase"} //for the first lightsource
            ZWrite Off
            Blend SrcAlpha OneMinusSrcAlpha
            CGPROGRAM
            #pragma vertex vert  
            #pragma geometry geom
            #pragma fragment frag 

            #include "AvatarVisualIndicator.cginc"
            
            fixed _Opacity;
            fixed _Saturation;

            //sampler2D _MainTex;
            //float4 _MainTex_ST;

            float4 _FillColor, _WireColor, _ScanColor;
            float _WireWidth, _Scan, _ScanThickness, _Lerp, _BlendWidth, _LowestWorldPosition, _MomentaryBodyHeight;

            ////////////////////////////////
            // In the origin avatar shader, we use struct v2f(vertex to fragment), here we use an additional geometry shader to render the mesh,
            // and we add struct v2g and g2f in AvatarVisualIndicator.cginc
            v2g vert(appdata v)
            {
                v2g o;
                o.vertex = UnityObjectToClipPos(v.vertex); //position of vertex?
                o.uv.xy = TRANSFORM_TEX(v.uv, _MainTex);
                o.uv.z = v.vertex.x;
                //o.worldPos = v.vertex;
                o.worldPos = mul(unity_ObjectToWorld, v.vertex);//world position of vertex
                o.localHeightPos = v.vertex;//local position of vertex

                o.normal = normalize(mul(float4(v.normal, 0.0), unity_WorldToObject).xyz);
                o.color = v.color;
                return o;
            }

            // geometry shader, compute the triangles
            [maxvertexcount(3)]
            void geom(triangle v2g IN[3], inout TriangleStream < g2f > triStream)
            {
                float2 p0 = IN[0].vertex.xy / IN[0].vertex.w;
                float2 p1 = IN[1].vertex.xy / IN[1].vertex.w;
                float2 p2 = IN[2].vertex.xy / IN[2].vertex.w;

                float2 v0 = p2 - p1;
                float2 v1 = p2 - p0;
                float2 v2 = p1 - p0;
                //triangles area
                float area = abs(v1.x * v2.y - v1.y * v2.x);

                
                g2f OUT;
                OUT.normal = IN[0].normal;//
                OUT.worldPos = IN[0].worldPos;//using world position of vertex
                OUT.localHeightPos = IN[0].localHeightPos;//using world position of vertex
                OUT.color = IN[0].color;//
                OUT.vertex = IN[0].vertex;
                OUT.uv = IN[0].uv;
                OUT.dist = float3(area / length(v0), 0, 0);
                triStream.Append(OUT);

                OUT.normal = IN[1].normal;//
                OUT.worldPos = IN[1].worldPos;//
                OUT.localHeightPos = IN[1].localHeightPos;//
                OUT.color = IN[1].color;//
                OUT.vertex = IN[1].vertex;
                OUT.uv = IN[1].uv;
                OUT.dist = float3(0, area / length(v1), 0);
                triStream.Append(OUT);

                OUT.normal = IN[2].normal;//
                OUT.worldPos = IN[2].worldPos;//
                OUT.localHeightPos = IN[2].localHeightPos;//
                OUT.color = IN[2].color;//
                OUT.vertex = IN[2].vertex;
                OUT.uv = IN[2].uv;
                OUT.dist = float3(0, 0, area / length(v2));
                triStream.Append(OUT);
            }


            fixed4 frag(g2f i, fixed facing : VFACE) : COLOR //SV_Target
            {
                float3 N = normalize(i.normal);
                float3 V = _WorldSpaceCameraPos - i.worldPos.xyz; 
                
                //clip(i.worldPos.y-_Scan);
                //-----------------------Below same as the orginal AvararShader--------------------------//
                if(_useNormalMap)
                {
                    N = perturb_normal(N, V, i.uv);
                }

                fixed4 col = fixed4(1, 0, 0, 1);
                float3 color_tmp;
                bool useAmbient = true; //ambient light is computed for the base light only
                float4 material = float4(_ambientIntensity, _diffuseIntensity, _specularIntensity, _shininess);

                if(_useLighting)
                {
                    col = _frontColor;
                    if(_useVertexColor)
                    {
                        col = i.color;
                    }

                    if(facing)
                    {
                        phong_lighting_g2f(N, material, col.rgb, i, color_tmp, useAmbient);
                        col = fixed4(color_tmp, 1);
                    }
                    else
                    {
                        phong_lighting_g2f(-N, material, _backColor.rgb, i, color_tmp, useAmbient);
                    }
                }
                else
                {
                    if(_useVertexColor)
                    {
                        col = i.color;
                    }
                    else 
                    {
                        if(_useTexture)
                        {
                            color_tmp = tex2D(_MainTex, i.uv).xyz;
                            col = fixed4(color_tmp, 1);
                        }
                        else
                        {
                            color_tmp = _frontColor;
                            col = fixed4(color_tmp, 1);
                        }
                    }
                }

                col = clamp(col, 0.0, 1.0);
                col.rgb = lerp(dot(col.rgb, float3(0.2125, 0.7154, 0.0721)), col.rgb, _Saturation);
                //fixed4 colgrey = (grey, grey, grey, 1);
                //col = lerp(colgrey, col, _Saturation);
                col.a *= _Opacity;
                //-----------------------Above same as the orginal AvararShader--------------------------//

                if (_useScan) {
                    // render the polygon wire 
                    fixed4 col_Wire;
                    float d = min(i.dist.x, min(i.dist.y, i.dist.z));

                    _WireColor = _ScanColor;
                    _FillColor = (_ScanColor)/2.0;

                    col_Wire = d < _WireWidth ? _WireColor : _FillColor;

                    // blend the col_Wire with the col for texture, _BlendWidth can control the blending area
                    //half blendValue = smoothstep(-_BlendWidth, 0, i.worldPos.y-_Scan);


                    //local hight of object based on world coordinates
                    float localHeightPos = i.worldPos.y - _LowestWorldPosition;

                    float currentScanHeight = _Scan * _MomentaryBodyHeight;

                    float materialChangePos = localHeightPos - currentScanHeight;

                    float heightClamp = smoothstep(0.0, 1.0, (materialChangePos * 0.8) / _BlendWidth + 0.5);

                    //Color blending under the scan
                    float scanSize = _MomentaryBodyHeight * _ScanThickness;

                    float spaceUnderScan = currentScanHeight - scanSize;

                    float underScanHeightClamp = smoothstep(0.0, 1.0, (materialChangePos * 0.8 + scanSize) / _BlendWidth + 0.5);

                    float4 returnColor = (localHeightPos > spaceUnderScan) ? lerp(col_Wire, col, heightClamp) : lerp(col, col_Wire, underScanHeightClamp);

                    return returnColor;

                    /*
                    // 6.) The LIGHT_ATTENUATION samples the shadowmap (using the coordinates calculated by TRANSFER_VERTEX_TO_FRAGMENT
                    // and stored in the structure defined by LIGHTING_COORDS), and returns the value as a float.
                    float attenuation = LIGHT_ATTENUATION(i);
                    //from here we start to compute the ambient
                    i.worldNorm = normalize(i.worldNorm);
                    half3 ambient = ShadeSH9(float4(i.worldNorm, 1));
                    //attenuation = clamp(attenuation, 0.3, 1.0);
                    //return col*attenuation;
                    return returnCol * (attenuation + half4(ambient, 1));
                    */
                }
                else {
                    return col;

                    /*
                    // 6.) The LIGHT_ATTENUATION samples the shadowmap (using the coordinates calculated by TRANSFER_VERTEX_TO_FRAGMENT
                    // and stored in the structure defined by LIGHTING_COORDS), and returns the value as a float.
                    float attenuation = LIGHT_ATTENUATION(i);
                    //from here we start to compute the ambient
                    i.worldNorm = normalize(i.worldNorm);
                    half3 ambient = ShadeSH9(float4(i.worldNorm, 1));
                    //attenuation = clamp(attenuation, 0.3, 1.0);
                    //return col*attenuation;
                    return col * (attenuation + half4(ambient, 1));
                    */
                }
                
            }
            ENDCG

            
        }

        Pass {
            // add same code for this pass

            Tags {"LightMode" = "ForwardAdd"} //for every other lightsource
            ZWrite Off
			//Blend SrcAlpha OneMinusSrcAlpha
            Blend One One
            CGPROGRAM
            #pragma vertex vert
            #pragma fragment frag
            #include "AvatarVisualIndicator.cginc"

            float4 _FillColor, _WireColor, _ScanColor;
            float _WireWidth, _Scan, _ScanThickness, _Lerp, _BlendWidth, _LowestWorldPosition, _MomentaryBodyHeight;

            fixed _Opacity;

            v2g vert(appdata v)
            {
                v2g o;
                o.vertex = UnityObjectToClipPos(v.vertex);
                o.uv.xy = TRANSFORM_TEX(v.uv, _MainTex);
                o.uv.z = v.vertex.x;
                //o.worldPos = v.vertex;
                o.worldPos = mul(unity_ObjectToWorld, v.vertex);
                o.localHeightPos = v.vertex;
    
                o.normal = normalize(mul(float4(v.normal, 0.0), unity_WorldToObject).xyz);
                o.color = v.color;
                return o;
            }

            [maxvertexcount(3)]
            void geom(triangle v2g IN[3], inout TriangleStream < g2f > triStream)
            {
                float2 p0 = IN[0].vertex.xy / IN[0].vertex.w;
                float2 p1 = IN[1].vertex.xy / IN[1].vertex.w;
                float2 p2 = IN[2].vertex.xy / IN[2].vertex.w;

                float2 v0 = p2 - p1;
                float2 v1 = p2 - p0;
                float2 v2 = p1 - p0;
                //triangles area
                float area = abs(v1.x * v2.y - v1.y * v2.x);

                //
                g2f OUT;
                OUT.normal = IN[0].normal;//
                OUT.worldPos = IN[0].worldPos;//
                OUT.localHeightPos = IN[0].localHeightPos;//
                OUT.color = IN[0].color;//
                OUT.vertex = IN[0].vertex;
                OUT.uv = IN[0].uv;
                OUT.dist = float3(area / length(v0), 0, 0);
                triStream.Append(OUT);

                OUT.normal = IN[1].normal;//
                OUT.worldPos = IN[1].worldPos;//
                OUT.localHeightPos = IN[1].localHeightPos;//
                OUT.color = IN[1].color;//
                OUT.vertex = IN[1].vertex;
                OUT.uv = IN[1].uv;
                OUT.dist = float3(0, area / length(v1), 0);
                triStream.Append(OUT);

                OUT.normal = IN[2].normal;//
                OUT.worldPos = IN[2].worldPos;//
                OUT.localHeightPos = IN[2].localHeightPos;//
                OUT.color = IN[2].color;//
                OUT.vertex = IN[2].vertex;
                OUT.uv = IN[2].uv;
                OUT.dist = float3(0, 0, area / length(v2));
                triStream.Append(OUT);
            }

            fixed4 frag(g2f i, fixed facing : VFACE) : COLOR //SV_Target
            {
                float3 N = normalize(i.normal);
                float3 V = _WorldSpaceCameraPos - i.worldPos.xyz; 

                if(_useNormalMap)
                {
                    N = perturb_normal(N, V, i.uv);
                }

                fixed4 col = fixed4(0, 0, 0, 1);
                float3 color_tmp = fixed4(0, 0, 0, 1);
                bool useAmbient = false; //ambient light is computed for the base light only
                float4 material = float4(_ambientIntensity, _diffuseIntensity, _specularIntensity, _shininess);
                if(_useLighting)
                {
                    col = _frontColor;
                    if(_useVertexColor)
                    {
                        col = i.color;
                    }

                    if(facing)
                    {
                        phong_lighting_g2f(N, material, col, i, color_tmp, useAmbient);
                        col = fixed4(color_tmp, 1);
                    }
                    else
                    {
                        phong_lighting_g2f(-N, material, _backColor, i, color_tmp, useAmbient);
                    }
                }
                col = clamp(col, 0.0, 1.0); 
                col.a *= _Opacity;


                if (_useScan) {
                    // render the polygon wire 
                    fixed4 col_Wire;
                    float d = min(i.dist.x, min(i.dist.y, i.dist.z));

                    _WireColor = _ScanColor;

                    col_Wire = d < _WireWidth ? _WireColor : _FillColor;

                    // blend the col_Wire with the col for texture, _BlendWidth can control the blending area
                    //half blendValue = smoothstep(-_BlendWidth, 0, i.worldPos.y-_Scan);


                    //local hight of object based on world coordinates
                    float localHeightPos = i.worldPos.y - _LowestWorldPosition;

                    float currentScanHeight = _Scan * _MomentaryBodyHeight;

                    float materialChangePos = localHeightPos - currentScanHeight;

                    float heightClamp = smoothstep(0.0, 1.0, (materialChangePos * 0.8) / _BlendWidth + 0.5);

                    //Color blending under the scan
                    float scanSize = _MomentaryBodyHeight * _ScanThickness;

                    float spaceUnderScan = currentScanHeight - scanSize;

                    float underScanHeightClamp = smoothstep(0.0, 1.0, (materialChangePos * 0.8 + scanSize) / _BlendWidth + 0.5);

                    float4 returnColor = (localHeightPos > spaceUnderScan) ? lerp(col_Wire, col, heightClamp) : lerp(col, col_Wire, underScanHeightClamp);

                    return returnColor;
                    /*
                    // 6.) The LIGHT_ATTENUATION samples the shadowmap (using the coordinates calculated by TRANSFER_VERTEX_TO_FRAGMENT
                    // and stored in the structure defined by LIGHTING_COORDS), and returns the value as a float.
                    float attenuation = LIGHT_ATTENUATION(i);
                    //from here we start to compute the ambient
                    i.worldNorm = normalize(i.worldNorm);
                    half3 ambient = ShadeSH9(float4(i.worldNorm, 1));
                    //attenuation = clamp(attenuation, 0.3, 1.0);
                    //return col*attenuation;
                    return returnCol * (attenuation + half4(ambient, 1));
                    */
                }
                else {
                    return col;

                    /*
                    // 6.) The LIGHT_ATTENUATION samples the shadowmap (using the coordinates calculated by TRANSFER_VERTEX_TO_FRAGMENT
                    // and stored in the structure defined by LIGHTING_COORDS), and returns the value as a float.
                    float attenuation = LIGHT_ATTENUATION(i);
                    //from here we start to compute the ambient
                    i.worldNorm = normalize(i.worldNorm);
                    half3 ambient = ShadeSH9(float4(i.worldNorm, 1));
                    //attenuation = clamp(attenuation, 0.3, 1.0);
                    //return col*attenuation;
                    return col * (attenuation + half4(ambient, 1));
                    */
                }
            }
            ENDCG
        }
    }
    FallBack "Diffuse" // what will happen if delete this line?
}